"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2018 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const process = require("process");
const child_process_promise_1 = require("child-process-promise");
const program = require("commander");
const inquirer = require("inquirer");
const request = require("request-promise-native");
const packageJson = require("../../package.json");
const config_1 = require("../config");
const gcloud_1 = require("../gcloud");
const keymanager_1 = require("../keymanager");
const GITHUB_API_BASE = 'https://api.github.com';
function makeGithubRequest(uri, body, token, method = 'POST') {
    return __awaiter(this, void 0, void 0, function* () {
        return request({
            method,
            uri,
            body,
            json: true,
            headers: {
                'User-Agent': 'cloudbuild-github',
            },
            auth: {
                user: 'token',
                pass: token,
            },
        });
    });
}
function deploy() {
    return __awaiter(this, void 0, void 0, function* () {
        const ui = new inquirer.ui.BottomBar();
        const projectId = yield gcloud_1.googleApis.getProjectId();
        const webhookSecret = yield keymanager_1.keyManager.getWebhookSecret();
        ui.log.write('Deploying cloud functions.');
        if (program.delete) {
            try {
                yield child_process_promise_1.spawn('gcloud', ['--quiet', 'beta', 'functions', 'delete', 'cloudbuildGithubWebhook'], { stdio: 'inherit' });
            }
            catch (err) {
                ui.log.write('Could not delete cloudbuildGithubWebhook, skipping.');
            }
            try {
                yield child_process_promise_1.spawn('gcloud', ['--quiet', 'beta', 'functions', 'delete', 'cloudbuildGithubNotifier'], { stdio: 'inherit' });
            }
            catch (err) {
                ui.log.write('Could not delete cloudbuildGithubNotifier, skipping.');
            }
        }
        yield child_process_promise_1.spawn('gcloud', [
            'beta',
            'functions',
            'deploy',
            'cloudbuildGithubWebhook',
            '--trigger-http',
        ], { stdio: 'inherit' });
        yield child_process_promise_1.spawn('gcloud', [
            'beta',
            'functions',
            'deploy',
            'cloudbuildGithubNotifier',
            '--trigger-topic',
            'cloud-builds',
        ], { stdio: 'inherit' });
        const webhookUrl = `https://us-central1-${projectId}.cloudfunctions.net/cloudbuildGithubWebhook`;
        yield Promise.all(Object.keys(config_1.default.repos).map((repoName) => __awaiter(this, void 0, void 0, function* () {
            const repoToken = yield keymanager_1.keyManager.getGithubToken(repoName);
            ui.log.write(`Setting up repository webhook for ${repoName}`);
            const hooksUri = `${GITHUB_API_BASE}/repos/${repoName}/hooks`;
            const existingHooks = yield makeGithubRequest(hooksUri, null, repoToken, 'GET');
            yield Promise.all(existingHooks
                .filter((h) => h.config.url === webhookUrl)
                .map((h) => __awaiter(this, void 0, void 0, function* () { return makeGithubRequest(`${hooksUri}/${h.id}`, null, repoToken, 'DELETE'); })));
            const hook = {
                name: 'web',
                config: {
                    url: webhookUrl,
                    content_type: 'json',
                    secret: webhookSecret,
                },
                events: ['pull_request'],
            };
            return makeGithubRequest(hooksUri, hook, repoToken);
        })));
        ui.log.write('Done!');
    });
}
program
    .version(packageJson.version)
    .option('--delete', 'Delete existing functions first')
    .parse(process.argv);
deploy().then(() => process.exit(), (err) => {
    console.error('Unexpected error.', err);
    process.exit(1);
});
//# sourceMappingURL=cloudbuild-cli-deploy.js.map